from urllib.parse import urljoin

import requests
from .exceptions import PortainerError


class Portainer:
    FROM_VERSION = (0, 0)
    URL_STACKS = "api/endpoints/1/stacks"
    URL_STACK = "api/stacks/{stack_id}"

    def __init__(self, host: str):
        self.token = None
        self.host = host

    def login(self, username: str, password: str):
        """
        Logs in to the portainer API
        :param username:  Portainer username
        :param password: Portainer password
        :return: True if authentication was successful, False if not
        """
        data = {"Username": username, "Password": password}
        r = self.request("api/auth", method="POST", data=data)
        self.token = r.get("jwt")
        return bool(self.token)

    def get_stacks(self) -> dict:
        """
        Get all stacks on this portainer
        :return: a list of dicts
        """
        return self.request(self.URL_STACKS)

    def get_stack(self, stack_id) -> dict:
        """
        Get the stack by Id. Id might be a string or an int depending
        on the Portainer version
        :param stack_id:
        :return: A dict containing information about the stack
        :raises LookupError:
        """
        return self.request(self.URL_STACK.format(stack_id=stack_id))

    def stack_with_name(self, name) -> dict:
        stacks = self.get_stacks()
        for stack in stacks:
            if stack.get("Name") == name:
                return stack
        raise LookupError("No stack with name '{}'".format(name))

    def get_env_vars(self, stack_id) -> dict:
        response = self.get_stack(stack_id)
        return {item["name"]: item["value"] for item in response["Env"]}

    def update_stack(
        self,
        stack_id: str,
        stack_file_content: str,
        env_vars: dict = None,
        prune: bool = False,
    ) -> dict:
        url = self.URL_STACK.format(stack_id=stack_id)
        data = {
            "StackFileContent": stack_file_content,
            "Prune": prune,
            "Env": [{"name": k, "value": v} for k, v in env_vars.items()],
        }
        return self.request(url, method="PUT", data=data)

    def update_stack_with_file(
        self,
        stack_id: str,
        stack_file_path: str,
        env_vars: dict = None,
        prune: bool = False,
    ) -> dict:
        with open(stack_file_path, "r") as f:
            stack_file = f.read()
        return self.update_stack(stack_id, stack_file, env_vars=env_vars, prune=prune)

    def endpoints(self):
        return self.request("api/endpoints")

    def request(self, path: str, method: str = "GET", data: dict = None) -> dict:
        url = urljoin(self.host, path)
        headers = {}
        if self.token:
            headers["Authorization"] = "Bearer {}".format(self.token)
        response = requests.request(method, url, json=data, headers=headers)
        if response.status_code == requests.codes.ok:
            return response.json()
        else:
            raise PortainerError(response=response)



from .portainer_base import Portainer
from .portainer_1_17 import Portainer_1_17
from .portainer_1_18 import Portainer_1_18


def get_closest_api_version(version_string):
    """
    Gets latest API version that matches the version string.
    Any new versions should be added to `all_versions` at the top.
    """
    all_versions = (
        Portainer_1_18,
        Portainer_1_17,
    )
    version = [int(i) for i in version_string.split('.')][:2]

    for portainer in all_versions:
        from_version = getattr(portainer, 'FROM_VERSION')
        if from_version[0] < version[0]:
            return portainer
        elif from_version[0] == version[0] and from_version[1] <= version[1]:
            return portainer

    # No suitable version found!
    return None



# -*- coding: utf-8 -*-

"""Main module."""

from urllib.parse import urljoin
from .api_versions import Portainer
import requests

from . import api_versions


def portainer_for_host(host) -> Portainer:
    """
    Finds and returns the right Portainer class for the version
    running on this host
    """
    url = urljoin(host, "api/status")
    r = requests.request("GET", url).json()
    return api_versions.get_closest_api_version(r.get("Version"))(host)



import json


class PortainerError(IOError):
    def __init__(self, *args, **kwargs):
        response = kwargs.pop("response", None)
        try:
            self.message = response.json()['err']
        except json.JSONDecodeError:
            self.message = response.content.decode("utf-8")
        super(PortainerError, self).__init__(*args, **kwargs)



from typing import Union

import click
from click import option
from colorama import init as colorama_init

from portainer_py import portainer_for_host, PortainerError

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.group(context_settings=CONTEXT_SETTINGS)
def cli():
    colorama_init()


def show_error(message: Union[str, Exception], stop: bool = False):
    click.secho("⨉ Error: {}".format(message), fg="red")
    if stop:
        print()
        exit(1)


@cli.command()
@option(
    "-f",
    "--stackfile",
    help="Path to the stackfile.yml",
    envvar="PORTAINER_STACKFILE",
    type=click.Path(exists=True),
    required=True,
)
@option(
    "-h", "--host", help="URL to Portainer host", envvar="PORTAINER_HOST", required=True
)
@option(
    "-u",
    "--user",
    help="Portainer username",
    envvar="PORTAINER_USERNAME",
    required=True,
)
@option(
    "-p",
    "--password",
    help="Portainer password",
    envvar="PORTAINER_PASSWORD",
    prompt="Portainer password",
    hide_input=True,
)
@option(
    "-n",
    "--stackname",
    help="Name of the Portainer stack",
    envvar="PORTAINER_STACK_NAME",
    required=True,
)
@option("-e", "--env", help="Environment variables to add to the stack", multiple=True)
@option(
    "--prune-env",
    help="If true, will remove existing environment variables from the stack",
    default=False,
    is_flag=True,
)
@option(
    "--prune-stack",
    help="Prune services that are no longer referenced",
    default=False,
    is_flag=True,
)
def deploy(stackfile, host, user, password, stackname, env, prune_env, prune_stack):
    """
    Update and deploy a portainer stack.

    To ease automation, this utility also support using environment variables
    instead of using these command line options:

    \b
    --user       PORTAINER_USERNAME
    --password   PORTAINER_PASSWORD
    --host       PORTAINER_HOST
    --stackname  PORTAINER_STACK_NAME
    --stackfile  PORTAINER_STACKFILE
    """

    portainer = portainer_for_host(host)

    try:
        portainer.login(user, password)
    except PortainerError as err:
        show_error(err.message, stop=True)

    try:
        stack = portainer.stack_with_name(stackname)
    except LookupError as err:
        show_error(err, stop=True)

    # Merge existing env vars on the stack with the supplied ones
    existing_env_vars = {} if prune_env else portainer.get_env_vars(stack["Id"])
    env_vars = {k: v for k, v in (item.split("=", 1) for item in env)}

    try:
        portainer.update_stack_with_file(
            stack["Id"],
            stackfile,
            env_vars={**existing_env_vars, **env_vars},
            prune=prune_stack,
        )
    except PortainerError as err:
        show_error(err.message, stop=True)
    except FileNotFoundError as err:
        show_error(err, stop=True)

    click.secho("√ Stack successfully updated!", fg="green")
    print()


if __name__ == "__main__":
    cli()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytest

from portainer_py.api_versions import (
    get_closest_api_version,
    Portainer_1_17,
    Portainer_1_18,
)


class TestVersionFinder:
    @pytest.mark.parametrize(
        "version,portainer_class",
        [
            ("0.0", Portainer_1_17),
            ("1.0", Portainer_1_17),
            ("1.0.999", Portainer_1_17),
            ("1.17", Portainer_1_17),
            ("1.17.1", Portainer_1_17),
            ("1.17.999", Portainer_1_17),
            ("1.18", Portainer_1_18),
            ("1.18.0", Portainer_1_18),
            ("1.18.999", Portainer_1_18),
            ("1.299", Portainer_1_18),
            ("2.000", Portainer_1_18),
            ("10.0.1", Portainer_1_18),
        ],
    )
    def test_version_low(self, version, portainer_class):
        assert get_closest_api_version(version) == portainer_class



from io import open

from setuptools import find_packages, setup

with open('portainer_py/__init__.py', 'r') as f:
    for line in f:
        if line.startswith('__version__'):
            version = line.strip().split('=')[1].strip(' \'"')
            break
    else:
        version = '0.0.1'

with open('README.rst', 'r', encoding='utf-8') as f:
    readme = f.read()

REQUIRES = ["requests", "click", "colorama"]

setup(
    name='portainer-py',
    version=version,
    description='',
    long_description=readme,
    long_description_content_type='text/x-rst; charset=UTF-8',
    author='Bob Vork',
    author_email='bob@theothersonline.eu',
    maintainer='Bob Vork',
    maintainer_email='bob@theothersonline.eu',
    url='https://github.com/_/portainer-py',
    license='MIT',

    keywords=[
        '',
    ],

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],

    install_requires=REQUIRES,
    tests_require=['coverage', 'pytest'],

    packages=find_packages(),
    py_modules=["portainer_py"],
    entry_points="""
    [console_scripts]
    portainerpy=portainer_py.cli:cli
    """
)
